<?php

namespace Teknisa\Libs\Service;

use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\InstanceProvider;
use Teknisa\Libs\Util\Utilities;

class Metadata
{
    const DS = DIRECTORY_SEPARATOR;
    const JSON_NAME_NOTFOUND = 'notfound';
    const JSON_NAME_UNAUTHORIZED = 'unauthorized';
    /**
     * @var Environment
     */
    private $environment;

    public function __construct(Environment $environment)
    {
        $this->environment = $environment;
    }

    public function findMetadata($jsonPath, $routePath = null)
    {
        $isJsonFragment = !$routePath;

        if (Utilities::isModule()) {
            $jsonMetadata = $this->getJsonMetadataByModule($jsonPath);
        } else {
            $jsonMetadata = $this->getJsonMetadataByProduct($jsonPath);
        }

        if (!$jsonMetadata) {
            return $this->getJsonMetadataByProduct(self::JSON_NAME_NOTFOUND);
        }

        if (!$isJsonFragment && $this->unauthorizedAccessToJsonMetadata($routePath)) {
            return $this->getJsonMetadataByProduct(self::JSON_NAME_UNAUTHORIZED);
        }

        $notAuthorized = $this->filterMetadataByAcl($jsonMetadata);

        if ($notAuthorized && $isJsonFragment) {
            return [];
        }

        if ($notAuthorized && !$isJsonFragment) {
            return $this->getJsonMetadataByProduct(self::JSON_NAME_UNAUTHORIZED);
        }

        return $jsonMetadata;
    }

    /**
     * @param $metadata
     * @return bool
     */
    private function filterMetadataByAcl(&$metadata): bool
    {
        foreach ($metadata as $key => &$data) {
            if (is_array($data)) {
                $notAuthorized = $this->filterMetadataByAcl($data);
                if ($notAuthorized) {
                    unset($metadata[$key]);
                    $metadata = array_values($metadata);
                }
            } else {
                /* Examples:
                    1) "addButton":    "ACL_LABEL|LABEL1,LABEL2"          -> return boolean: true or false
                    2) "deleteButton": "ACL_LABEL|LABEL1|selection,none"  -> return string: selection or none
                    3) "intValue":     "ACL_LABEL|LABEL1,LABEL2|1,2|int"  -> return int: 1 or 2
                    4) "floatValue":   "ACL_LABEL|LABEL1|1.01,2.99|float" -> return float: 1.01 or 2.99
                */
                if(is_string($data) && strpos($data, 'ACL_LABEL') !== false) {
                    $labelInfo = explode('|', $data);
                    $labels = $labelInfo[1];
                    $values = isset($labelInfo[2]) ? explode(',', $labelInfo[2]) : array(true, false);
                    $data = $this->checkLabelInSession($labels) ? $values[0] : $values[1];
                    $valueType = isset($labelInfo[3]) ? $labelInfo[3] : null;
                    if($valueType == 'int') {
                        $data = intval($data);
                    } else if($valueType == 'float') {
                        $data = floatval($data);
                    }
                } else if ($key === 'aclLabel' && !$this->checkLabelInSession($data)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * @param $aclLabel
     * @return bool
     */
    private function checkLabelInSession($aclLabel): bool
    {
        $isAdm = InstanceProvider::getSession()->get('ACL_ADMINISTRATOR');
        $userAclLabels = InstanceProvider::getSession()->get('USER_ACL_LABELS');
        if($isAdm || is_null($userAclLabels)) {
            return true;
        }

        $labels = explode(',', $aclLabel);
        foreach($labels as $label) {
            if(in_array($label, $userAclLabels)) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param $name
     * @return string
     */
    private function getJsonMetadataPathByProduct($name): string
    {
        return Utilities::getProductBasePath() . self::DS . Utilities::getJsonMetadataPathByProductParameter() . self::DS . $name . '.json';
    }

    /**
     * @param $name
     * @return string
     * Mudar método posteriormente
     */
    private function getJsonMetadataPathByModule($name): string
    {
        return Utilities::getProductBasePath() . self::DS . Utilities::getJsonMetadataPathByModuleParameter() . self::DS . $name . '.json';
    }


    /**
     * @param $jsonPath
     * @return mixed
     */
    private function getJsonMetadataByProduct($jsonPath)
    {
        $jsonMetadataPath = $this->getJsonMetadataPathByProduct($jsonPath);
        return @json_decode(file_get_contents($jsonMetadataPath), true);
    }

    private function getJsonMetadataByModule($jsonPath)
    {
        $jsonMetadataPath = $this->getJsonMetadataPathByModule($jsonPath);
        return @json_decode(file_get_contents($jsonMetadataPath), true);
    }

    /**
     * @param $routePath
     * @return bool
     */
    private function unauthorizedAccessToJsonMetadata($routePath): bool
    {
        $menuRoutes = InstanceProvider::getSession()->get('MENU_ROUTES');
        return isset($menuRoutes[$routePath]) && !$menuRoutes[$routePath];
    }

    /**
     * @param $routePath
     * @return bool
     */
    private function hasIndexInMenuRoutesSession($routePath): bool
    {
        $menuRoutes = InstanceProvider::getSession()->get('MENU_ROUTES');
        return isset($menuRoutes[$routePath]);
    }
}
